/**
 * 
 */
package de.mcs.utils;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

/**
 * @author w.klaas
 *
 */
public class TestVersion {

  class GreatLessVersionTupel {
    Version greatVersion;
    Version lessVersion;

    public GreatLessVersionTupel(Version greatVersion, Version lessVersion) {
      this.greatVersion = greatVersion;
      this.lessVersion = lessVersion;
    }
  }

  class VersionFormat {
    String versionString;
    int major;
    int minor;
    int release;
    int build;
    private String versionState;

    public VersionFormat(String versionString, int major, int minor, int release, int build) {
      this.versionString = versionString;
      this.build = build;
      this.major = major;
      this.minor = minor;
      this.release = release;
    }

    public VersionFormat(String versionString, int major, int minor, int release, int build, String versionStatus) {
      this.versionString = versionString;
      this.build = build;
      this.major = major;
      this.minor = minor;
      this.release = release;
      this.versionState = versionStatus;
    }

    public boolean hasVersionState() {
      return (versionState != null) && !versionState.equals("");
    }
  }

  VersionFormat[] versionFormats = { new VersionFormat("1", 1, -1, -1, -1), new VersionFormat("1.0", 1, 0, -1, -1),
      new VersionFormat("1.0.0", 1, 0, 0, -1), new VersionFormat("1.0.0.0", 1, 0, 0, 0),
      new VersionFormat("2.1", 2, 1, -1, -1), new VersionFormat("2.3.5.0054", 2, 3, 5, 54),
      new VersionFormat("2 - SNAPSHOT", 2, -1, -1, -1, "SNAPSHOT"),
      new VersionFormat("2.1- RELEASE", 2, 1, -1, -1, "RELEASE"),
      new VersionFormat("2.1.7 -SNAPSHOT", 2, 1, 7, -1, "SNAPSHOT"),
      new VersionFormat("2.1.4.5-VERSIONSTATE", 2, 1, 4, 5, "VERSIONSTATE"),
      new VersionFormat("2.1.4.5-", 2, 1, 4, 5, "") };

  /**
   * Test method for {@link de.mcs.utils.Version#Version(java.lang.String)}.
   */
  @Test
  public void testVersionString() {
    for (VersionFormat version : versionFormats) {
      Version versionUnderTest = new Version(version.versionString);
      assertEquals(version.major, versionUnderTest.getMajor());
      assertEquals(version.minor, versionUnderTest.getMinor());
      assertEquals(version.release, versionUnderTest.getRelease());
      assertEquals(version.build, versionUnderTest.getBuild());
      if (version.hasVersionState()) {
        assertTrue(versionUnderTest.hasVersionState());
        assertEquals(version.versionState, versionUnderTest.getVersionState());
      }
    }
  }

  /**
   * Test method for {@link de.mcs.utils.Version#incrementMajor()}.
   */
  @Test
  public void testIncrementAll() {
    for (VersionFormat versionFormat : versionFormats) {
      Version version = new Version(versionFormat.versionString);
      assertEquals(versionFormat.major, version.getMajor());
      version.incrementMajor();
      assertEquals(versionFormat.major + 1, version.getMajor());
      version.incrementMajor();
      assertEquals(versionFormat.major + 2, version.getMajor());

      assertEquals(versionFormat.minor, version.getMinor());
      version.incrementMinor();
      assertEquals(versionFormat.minor + 1, version.getMinor());
      version.incrementMinor();
      assertEquals(versionFormat.minor + 2, version.getMinor());

      assertEquals(versionFormat.release, version.getRelease());
      version.incrementRelease();
      assertEquals(versionFormat.release + 1, version.getRelease());
      version.incrementRelease();
      assertEquals(versionFormat.release + 2, version.getRelease());

      assertEquals(versionFormat.build, version.getBuild());
      version.incrementBuild();
      assertEquals(versionFormat.build + 1, version.getBuild());
      version.incrementBuild();
      assertEquals(versionFormat.build + 2, version.getBuild());
    }
  }

  /**
   * Test method for {@link de.mcs.utils.Version#decrementMajor()}.
   */
  @Test
  public void testDecrementAll() {
    for (VersionFormat versionFormat : versionFormats) {
      Version version = new Version(versionFormat.versionString);
      assertEquals(versionFormat.major, version.getMajor());
      version.decrementMajor();
      assertEquals(versionFormat.major - 1, version.getMajor());
      version.decrementMajor();
      assertEquals(versionFormat.major - 2, version.getMajor());

      assertEquals(versionFormat.minor, version.getMinor());
      version.decrementMinor();
      assertEquals(versionFormat.minor - 1, version.getMinor());
      version.decrementMinor();
      assertEquals(versionFormat.minor - 2, version.getMinor());

      assertEquals(versionFormat.release, version.getRelease());
      version.decrementRelease();
      assertEquals(versionFormat.release - 1, version.getRelease());
      version.decrementRelease();
      assertEquals(versionFormat.release - 2, version.getRelease());

      assertEquals(versionFormat.build, version.getBuild());
      version.decrementBuild();
      assertEquals(versionFormat.build - 1, version.getBuild());
      version.decrementBuild();
      assertEquals(versionFormat.build - 2, version.getBuild());
    }
  }

  /**
   * Test method for
   * {@link de.mcs.utils.Version#greaterThan(de.mcs.utils.Version)}.
   */
  @Test
  public void testGreaterLesserThanEquals() {
    GreatLessVersionTupel[] greatLessVersionTupels = new GreatLessVersionTupel[] {
        new GreatLessVersionTupel(new Version("2"), new Version("1")),
        new GreatLessVersionTupel(new Version("2.0"), new Version("1.0")),
        new GreatLessVersionTupel(new Version("2.0.0"), new Version("1.0.0")),
        new GreatLessVersionTupel(new Version("2.0.0.0"), new Version("1.0.0.0")),
        new GreatLessVersionTupel(new Version("2.1"), new Version("2.0")),
        new GreatLessVersionTupel(new Version("2.1"), new Version("2.0.0")),
        new GreatLessVersionTupel(new Version("2.1"), new Version("2.0.0.0")),
        new GreatLessVersionTupel(new Version("2.0.1"), new Version("2.0.0")),
        new GreatLessVersionTupel(new Version("2.0.1"), new Version("2.0.0.0")),
        new GreatLessVersionTupel(new Version("2.0.0.1"), new Version("2.0.0.0")),
        new GreatLessVersionTupel(new Version("2"), new Version("2-SNAPSHOT")),
        new GreatLessVersionTupel(new Version("2.0"), new Version("2.0-SNAPSHOT")),
        new GreatLessVersionTupel(new Version("2.1-SNAPSHOT"), new Version("2.0-SNAPSHOT")),
        new GreatLessVersionTupel(new Version("2.1-RELEASE"), new Version("2.1-SNAPSHOT")),
        new GreatLessVersionTupel(new Version("2"), new Version("1")) };

    for (GreatLessVersionTupel greatLessVersionTupel : greatLessVersionTupels) {
      Version lessVersion = greatLessVersionTupel.lessVersion;
      Version greatVersion = greatLessVersionTupel.greatVersion;

      assertTrue(greatVersion.greaterThan(lessVersion));
      assertFalse(lessVersion.greaterThan(greatVersion));

      assertTrue(lessVersion.lesserThan(greatVersion));
      assertFalse(greatVersion.lesserThan(lessVersion));

      assertFalse(greatVersion.equals(lessVersion));
      assertFalse(lessVersion.equals(greatVersion));

      assertTrue(greatVersion.equals(greatVersion));
      assertTrue(lessVersion.equals(lessVersion));
    }
  }

  @Test
  public void quickTest() {
    GreatLessVersionTupel greatLessVersionTupel = new GreatLessVersionTupel(new Version("2.1"),
        new Version("2-SNAPSHOT"));

    Version lessVersion = greatLessVersionTupel.lessVersion;
    Version greatVersion = greatLessVersionTupel.greatVersion;

    assertTrue(greatVersion.greaterThan(lessVersion));
    assertFalse(lessVersion.greaterThan(greatVersion));

    assertTrue(lessVersion.lesserThan(greatVersion));
    assertFalse(greatVersion.lesserThan(lessVersion));

    assertFalse(greatVersion.equals(lessVersion));
    assertFalse(lessVersion.equals(greatVersion));

    assertTrue(greatVersion.equals(greatVersion));
    assertTrue(lessVersion.equals(lessVersion));
  }
}
