/**
 * 
 */
package de.mcs.utils.caches;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import org.junit.Before;
import org.junit.Test;

import de.mcs.utils.StringUtils;

/**
 * @author wklaa_000
 *
 */
public class TestLocalCache {

  private class CacheEntry extends TimedCacheEntry {

    private boolean result;

    /**
     * @param key
     * @param b
     */
    private CacheEntry(String key, boolean b) {
      super(key);
      this.result = b;
    }

  }

  private static final int CACHE_INVALIDATION_TIME = 5 * 60 * 1000;

  private static final TimedCache<CacheEntry> localCache = new TimedCache<>();

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception {
    localCache.clear();
  }

  @Test
  public void test() throws InterruptedException {
    // fill up local cache
    System.out.println("fill up cache");
    for (int i = 0; i < 50000; i++) {
      String cacheKey = createCacheKey(i);

      CacheEntry entry = new CacheEntry(cacheKey, true);
      entry.setTime(System.currentTimeMillis() - CACHE_INVALIDATION_TIME);
      localCache.put(entry);
    }

    for (int i = 50000; i < 100000; i++) {
      String cacheKey = createCacheKey(i);

      CacheEntry entry = new CacheEntry(cacheKey, true);
      localCache.put(entry);
    }

    System.out.println("test cache");
    for (int i = 0; i < 100000; i++) {
      String cacheKey = createCacheKey(i);
      CacheEntry entry = localCache.get(cacheKey);
      assertNotNull(entry);

      if (i >= 50000) {
        assertTrue((entry.getTime() + CACHE_INVALIDATION_TIME) > System.currentTimeMillis());
      } else {
        assertFalse((entry.getTime() + CACHE_INVALIDATION_TIME) > System.currentTimeMillis());
      }

    }

    // boolean result = authenticateUser(r, username, password, config);

    localCache.asyncRemoveEntries(CACHE_INVALIDATION_TIME);
    localCache.asyncRemoveEntries(CACHE_INVALIDATION_TIME);

    assertEquals(1, localCache.getCount());
    Thread.sleep(1000);
    assertEquals(50000, localCache.size());

  }

  private String createCacheKey(int i) {
    String username = String.format("Hallo%d", i);
    String password = String.format("pwd%d", i);
    String cacheKey = StringUtils.md5String(String.format("%s_%s", username, password));
    return cacheKey;
  };

}
